import socket
import requests
import threading
import time

class Packets:
    @staticmethod
    def Constructor(data, type: str):
        return f"{type}|".encode() + "|".join(data).encode()
    
    @staticmethod
    def Deconstructor(data: str):
        splitted = data.decode().split("|")
        return (splitted[0], splitted[1:])
    
    @staticmethod
    def connect(fromip, toip, fromport, toport):
        return Packets.Constructor([fromip, toip, str(fromport), str(toport)], "CONNECT")
    
    @staticmethod
    def disconnect(fromip, toip):
        return Packets.Constructor([fromip, toip], "DISCONNECT")
    
class Utils:
    @staticmethod
    def MyIP():
        try:
            response = requests.get("https://api.ipify.org?format=json")
            public_ip = response.json()["ip"]
            return public_ip
        except requests.RequestException as e:
            print(f"[BGM ERROR] Failed to get public ip.")
            return None

class BGM_Client:
    def __init__(self):
        self.ConnectedAddress = ()
        self.ConnectedSocket = None
        self.running = False
        
        self.ReciveCallBacks = []
        
    def connect(self, ip: str, port: int):
        address = (ip, port)
        sock =  socket.socket(socket.AF_INET, socket.SOCK_DGRAM)

        connect_packet = Packets.connect(Utils.MyIP(), ip, 0, port)
        
        sock.sendto(connect_packet, address)
        
        self.ConnectedAddress = address
        self.ConnectedSocket = sock
        
        self.running = True
        
        thread = threading.Thread(target=self._reciveloop)
        thread.daemon = True
        thread.start()
        
        return sock

    def _reciveloop(self):
        if self.ConnectedSocket != None:
            while self.running:
                try:
                    data, addr = self.ConnectedSocket.recvfrom(1024)
                    for callback in self.ReciveCallBacks:
                        callback(Packets.Deconstructor(data), addr)
                except Exception as e:
                    pass
        else:
            print("[BGM ERROR] Connect first to start recive loop.")

    def send(self, data, packet):
        if self.ConnectedSocket != None:
            self.ConnectedSocket.sendto(Packets.Constructor(data, packet), self.ConnectedAddress)
        else:
            print("[BGM ERROR] Connect first to use send function.")
            
    def recive_event(self, callback):
        self.ReciveCallBacks.append(callback)
        
    def disconnect(self):
        if self.ConnectedSocket != None:
            time.sleep(0.2)
            disconnect_packet = Packets.disconnect(Utils.MyIP(), self.ConnectedAddress[0])
            self.ConnectedSocket.sendto(disconnect_packet, self.ConnectedAddress)
            self.ConnectedSocket.close()
            self.ConnectedSocket = None
            
            self.running = False
        else:
            print("[BGM ERROR] Connect first to use disconnect function.")
            


import socket

class Packets:
    @staticmethod
    def Constructor(data, type: str):
        return f"{type}|".encode() + "|".join(data).encode()
    
    @staticmethod
    def Deconstructor(data: str):
        splitted = data.decode().split("|")
        return (splitted[0], splitted[1:])

class BGM_Server:
    def __init__(self, host, port):
        self.Address = (host, port)
        self.Sock = None
        
        self.Clients = []
        
        self.ReciveCallBacks = []
    
    def recive_event(self, callback):
        self.ReciveCallBacks.append(callback)
    
    def start_server(self):
        self.Sock = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
        self.Sock.bind(self.Address)
        
        print(f"Server on {self.Address[0]}:{self.Address[1]}")
        
        while True:
            try:
                message, client_address = self.Sock.recvfrom(1024)
                
                message = Packets.Deconstructor(message)

                if message[0] == "CONNECT":
                    self.Clients.append(client_address)
                elif message[0] == "DISCONNECT":
                    self.Clients.remove(client_address)
                else:
                    for callback in self.ReciveCallBacks:
                        callback(message, client_address)
            except:
                pass
                
    def broadcast(self, message):
        for client in self.Clients:
            self.Sock.sendto(message, client)



from setuptools import setup, find_packages

setup(
    name="bgm-net",  # Unique name on PyPI
    version="0.1.0",  # Version number
    author="DanXvo",
    author_email="data.com0010@gmail.com",
    description="Python library that made for Multiplayer games.",
    long_description=open("README.md").read(),
    long_description_content_type="text/markdown",
    url="https://github.com/DanXvoIsMe/BGM",
    packages=find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "Operating System :: OS Independent",
    ],
    install_requires=[
        "requests",
    ],
    python_requires=">=3.6",
)
